﻿//
// NTFS - Myriadbits .NET NTFS library. 
// Fast MFT reader.
// Copyright (C) 2014 Myriadbits, Jochem Bakker
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// For more information, contact me at: info@myriadbits.com
//

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Myriadbits.NTFS
{
	public class DataBuffer
	{
		private byte[] _buffer = null;
		private DateTime _dateOffset = new DateTime(1601, 1, 1);

		public DataBuffer(byte[] buffer)
		{
			this._buffer = buffer;
		}

		public int Length { get { return this._buffer.Length; } }
		public byte GetByte(int offset) { return (byte) BitConverter.ToInt16(this._buffer, offset); }
		public short GetShort(int offset) { return BitConverter.ToInt16(this._buffer, offset); }
		public ushort GetUShort(int offset) { return BitConverter.ToUInt16(this._buffer, offset); }
		public int GetInt(int offset) { return BitConverter.ToInt32(this._buffer, offset); }
		public uint GetUInt(int offset) { return BitConverter.ToUInt32(this._buffer, offset); }
		public long GetLong(int offset) { return BitConverter.ToInt64(this._buffer, offset); }
		public ulong GetULong(int offset) { return BitConverter.ToUInt64(this._buffer, offset); }
		public string GetString(int offset, int length) { return System.Text.Encoding.Unicode.GetString(this._buffer, offset, length); }

		public void SetUShort(int offset, uint value) 
		{
			this._buffer[offset + 0] = (byte)(value & 0xFF);
			this._buffer[offset + 1] = (byte)((value >> 8) & 0xFF); 
		}


		public byte[] GetData(int offset, int length)
		{
			byte[] data = new byte[length];
			Buffer.BlockCopy(this._buffer, offset, data, 0, length);
			return data;
		}

		public DateTime GetDateTime(int offset) 
		{ 
			long ticks = BitConverter.ToInt64(this._buffer, offset); // In 100ns since 1-1-1601
			ticks += this._dateOffset.Ticks;
			return new DateTime(ticks);
		}

		public ulong GetVar(int offset, int length)
		{
			// Copy to the last part our temp array
			byte [] data = new byte[8];
			Buffer.BlockCopy(this._buffer, offset, data, 0, length);
			return BitConverter.ToUInt64(data, 0);
		}
	}
}
