﻿//
// NTFS - Myriadbits .NET NTFS library. 
// Fast MFT reader.
// Copyright (C) 2014 Myriadbits, Jochem Bakker
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// For more information, contact me at: info@myriadbits.com
//

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;


namespace Myriadbits.NTFS
{
	public class NTFSHeader : DataBufferReader
	{
		public NTFSHeader(DataBuffer buffer, int offset = 0)
			: base(buffer, 0, offset)
		{
		}

		public ushort BytesPerSect { get { return this.GetUShort(0x0B); } }
		public byte SectorsPerCluster { get { return this.GetByte(0x0D); } }
		public byte MediaDescriptor { get { return this.GetByte(0x15); } }
		public ushort SectorsPerTrack { get { return this.GetUShort(0x18); } }
		public ulong TotalSectors { get { return this.GetULong(0x28); } }
		public ulong MFT { get { return this.GetULong(0x30); } }
		public ulong MFTMirror { get { return this.GetULong(0x38); } }
		public uint ClustersPerMFTRecord { get { return this.GetUInt(0x40); } }
		public uint ClustersPerIndexRecord { get { return this.GetUInt(0x44); } }
		public ulong VolumeSerialNumber { get { return this.GetULong(0x48); } }

		public ulong BlockSize { get { return 128 * ((ulong)this.SectorsPerCluster) * ((ulong)this.BytesPerSect); } }
		public ulong BytesPerCluster { get { return ((ulong) this.BytesPerSect) * ((ulong) this.SectorsPerCluster); } }


		public ulong MFTOffset { get { return ((ulong)this.MFT) * ((ulong)this.BytesPerSect) * ((ulong)this.SectorsPerCluster); } }
		public ulong MFTMirrorOffset { get { return ((ulong)this.MFTMirror) * ((ulong)this.BytesPerSect) * ((ulong)this.SectorsPerCluster); } }

		public override string ToString()
		{
			return string.Format("NTFS-{0}", VolumeSerialNumber);
		}
	}
}
