﻿//
// NTFS - Myriadbits .NET NTFS library. 
// Fast MFT reader.
// Copyright (C) 2014 Myriadbits, Jochem Bakker
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// For more information, contact me at: info@myriadbits.com
//

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;


namespace Myriadbits.NTFS
{
	public class NTFSIndexRoot_IndexHeader : DataBufferReader
	{
		public NTFSIndexRoot_IndexHeader(DataBuffer buffer, int offset)
			: base(buffer, 0, offset)
		{
		}

		public uint OffsetToFirstIndexEntry { get { return this.GetUInt(0x00); } }
		public uint TotalSizeOfIndexEntries { get { return this.GetUInt(0x04); } }
		public uint AllocatedSizeOfIndexEntries { get { return this.GetUInt(0x08); } }
		public byte Flags { get { return this.GetByte(0x0C); } }

		public List<NTFSIndexRoot_IndexRecord> IndexRecords
		{
			get
			{
				List<NTFSIndexRoot_IndexRecord> records = new List<NTFSIndexRoot_IndexRecord>();
				int currentSize = 0;
				int currentOffset = (int) this.OffsetToFirstIndexEntry;
				while (currentSize < TotalSizeOfIndexEntries)
				{
					NTFSIndexRoot_IndexRecord record = new NTFSIndexRoot_IndexRecord(this._buffer, this._dataoffset + currentOffset);
					if (record.FileReferenceNumber == 0x00 || record.IndexFlags != 0x00 || record.SizeOfIndexEntry == 0x10)
						break; // Last entry
					currentOffset += record.SizeOfIndexEntry;
					currentSize += record.SizeOfIndexEntry;
					if ((record.FileFlags & 0x0000000010000000) == 0)
						records.Add(record);
				}
				return records;
			}
		}

		public override string ToString()
		{
			string recordString = string.Join(", ", this.IndexRecords.Select(a => a.ToString())); 
			return string.Format("$INDEX_HEADER: {0}", recordString);
		}
	}
}
