﻿//
// NTFS - Myriadbits .NET NTFS library. 
// Fast MFT reader.
// Copyright (C) 2014 Myriadbits, Jochem Bakker
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// For more information, contact me at: info@myriadbits.com
//

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;


namespace Myriadbits.NTFS
{
	class NTFSDataRuns
	{
		List<NTFSDataRun> _dataRuns = new List<NTFSDataRun>();

		/// <summary>
		/// Constructor, read the dataruns
		/// </summary>
		/// <param name="dataAttribute"></param>
		public NTFSDataRuns(NTFSAttribute dataAttribute)
		{
			if (dataAttribute != null)
			{
				// Parse the dataruns
				int offset = dataAttribute.NonResident_DatarunOffset;
				byte temp = dataAttribute.GetByte(offset);
				ulong dataOffset = 0;
				while (temp != 0)
				{
					// Read the lengths:
					int nobOffset = (temp >> 4) & 0x0F;
					int nobLength = temp & 0x0F;

					ulong dataLength = dataAttribute.GetVar(offset + 1, nobLength);
					dataOffset += dataAttribute.GetVar(offset + 1 + nobLength, nobOffset);

					// Found MFT, add the datarun
					_dataRuns.Add(new NTFSDataRun(dataOffset, dataLength));

					// Jump to the next datarun size marker
					offset += nobOffset + nobLength + 1;
					temp = dataAttribute.GetByte(offset);
				}
			}
		}


		/// <summary>
		/// Clear all dataruns
		/// </summary>
		public void Clear()
		{
			this._dataRuns.Clear();
		}


		/// <summary>
		/// Return the total size of all dataruns combined
		/// </summary>
		public ulong TotalSize 
		{
			get
			{
				ulong total = 0;
				foreach(NTFSDataRun dataRun in this._dataRuns)
					total += dataRun.Length;
				return total;
			} 
		}


		/// <summary>
		/// Return the list of dataruns
		/// </summary>
		public List<NTFSDataRun> DataRuns 
		{ 
			get
			{
				return this._dataRuns;
			}
		}
	}
}
